@@ -43,23 +43,47 @@
 #' @importFrom digest digest
 #' @name ClusterFuture-class
 #' @keywords internal
-ClusterFuture <- function(expr = NULL, envir = parent.frame(), substitute = FALSE, globals = TRUE, packages = NULL, local = !persistent, gc = FALSE, persistent = FALSE, workers = NULL, user = NULL, master = NULL, revtunnel = TRUE, homogeneous = TRUE, ...) {
+ClusterFuture <- function(expr = NULL, substitute = TRUE, envir = parent.frame(), globals = TRUE, packages = NULL, local = !persistent, persistent = FALSE, workers = NULL, user = NULL, master = NULL, revtunnel = TRUE, homogeneous = TRUE, ...) {
   if (substitute) expr <- substitute(expr)
+  
+  stop_if_not(is.logical(persistent), length(persistent) == 1L,
+              !is.na(persistent))
 
+  ## Global objects
+  gp <- getGlobalsAndPackages(expr, envir = envir, persistent = persistent, globals = globals)
+
+  args <- list(...)
+
+  ## Which '...' arguments should be passed to Future() and 
+  ## which should be passed to makeClusterPSOCK()?
+  future_args <- !is.element(names(args), makeClusterPSOCK_args())
+  
+  future <- do.call(MultiprocessFuture, args = c(list(expr = quote(gp$expr), substitute = FALSE, envir = envir, globals = gp$globals, packages = c(packages, gp$packages), local = local, node = NA_integer_, persistent = persistent), args[future_args]))
+
+  future <- do.call(as_ClusterFuture, args = c(list(future, workers = workers, user = user, master = master, revtunnel = revtunnel, homogeneous = homogeneous), args[!future_args]))
+
+  future
+}
+
+
+as_ClusterFuture <- function(future, workers = NULL, ...) {
   if (is.function(workers)) workers <- workers()
   if (is.null(workers)) {
     getDefaultCluster <- importParallel("getDefaultCluster")
     workers <- getDefaultCluster()
+    workers <- addCovrLibPath(workers)
   } else if (is.character(workers) || is.numeric(workers)) {
-    workers <- ClusterRegistry("start", workers = workers, user = user, master = master, revtunnel = revtunnel, homogeneous = homogeneous)
+    workers <- ClusterRegistry("start", workers = workers, ...)
   } else {
     workers <- as.cluster(workers)
+    workers <- addCovrLibPath(workers)
   }
   if (!inherits(workers, "cluster")) {
     stop("Argument 'workers' is not of class 'cluster': ", paste(sQuote(class(workers)), collapse = ", "))
   }
   stop_if_not(length(workers) > 0)
 
+
   ## Attached workers' session information, unless already done.
   ## FIXME: We cannot do this here, because it introduces a race condition
   ## where multiple similar requests may appear at the same time bringing
@@ -75,19 +99,14 @@ ClusterFuture <- function(expr = NULL, envir = parent.frame(), substitute = FALS
     attr(workers, "name") <- name
   }
 
-  ## Global objects
-  gp <- getGlobalsAndPackages(expr, envir = envir, persistent = persistent, globals = globals)
-  globals <- gp$globals
-  packages <- unique(c(packages, gp$packages))
-  expr <- gp$expr
-  gp <- NULL
+  future$workers <- workers
 
-  f <- MultiprocessFuture(expr = expr, envir = envir, substitute = FALSE, globals = globals, packages = packages, local = local, gc = gc, persistent = persistent, workers = workers, node = NA_integer_, version = "1.8", ...)
-  structure(f, class = c("ClusterFuture", class(f)))
+  future <- structure(future, class = c("ClusterFuture", class(future)))
+
+  future
 }
 
 
-
 #' @importFrom parallel clusterCall clusterExport
 #' @export
 run.ClusterFuture <- function(future, ...) {
@@ -122,18 +141,6 @@ run.ClusterFuture <- function(future, ...) {
   cl <- workers[node_idx]
 
 
-  ## WORKAROUND: When running covr::package_coverage(), the
-  ## package being tested may actually not be installed in
-  ## library path used by covr.  We here add that path iff
-  ## covr is being used. /HB 2016-01-15
-  if (is.element("covr", loadedNamespaces())) {
-    if (debug) mdebug("covr::package_coverage() workaround ...")
-    libPath <- .libPaths()[1]
-    clusterCall(cl, fun = function() .libPaths(c(libPath, .libPaths())))
-    if (debug) mdebug("covr::package_coverage() workaround ... DONE")
-  }
-
-
   ## (i) Reset global environment of cluster node such that
   ##     previous futures are not affecting this one, which
   ##     may happen even if the future is evaluated inside a
@@ -202,6 +209,7 @@ run.ClusterFuture <- function(future, ...) {
   invisible(future)
 }
 
+#' @importFrom parallelly connectionId isConnectionValid
 #' @export
 resolved.ClusterFuture <- function(x, run = TRUE, timeout = 0.2, ...) {
   workers <- x$workers
@@ -249,7 +257,7 @@ resolved.ClusterFuture <- function(x, run = TRUE, timeout = 0.2, ...) {
     connId <- connectionId(con)
     if (!is.na(connId) && connId < 0L) return(FALSE)
 
-    isValid <- isValidConnection(con)
+    isValid <- isConnectionValid(con)
     if (!isValid) {
       label <- x$label
       if (is.null(label)) label <- "<none>"
@@ -262,16 +270,27 @@ resolved.ClusterFuture <- function(x, run = TRUE, timeout = 0.2, ...) {
     if (.Platform$OS.type != "windows" && getRversion() < "3.4.3") {
       timeout <- round(timeout, digits = 0L)
     }
-    res <- socketSelect(list(con), write = FALSE, timeout = timeout)
+    maxCount <- 100L
+    count <- 0L
+    while (count < maxCount) {
+      ## Is there a message from the worker waiting?
+      res <- socketSelect(list(con), write = FALSE, timeout = timeout)
+      if (!res) break
 
-    if (res) {
-      ## It could be that the message available from the worker is something else
-      ## than a FutureResult, e.g. a condition.  Consider the future resolved,
-      ## only if a FutureResult was sent.
+      ## Receive it
       msg <- receiveMessageFromWorker(x)
+
+      ## If the message contains a FutureResult, then the future is resolved
+      ## and we are done here
       res <- inherits(msg, "FutureResult")
       msg <- NULL
-    }
+      if (res) break
+
+      ## If not, we received a condition that has already been signaled
+      ## by receiveMessageFromWorker().  However, it could be that there is
+      ## another condition messages available, so lets check again
+      count <- count + 1L
+    } ## while()
   } else if (inherits(node, "MPInode")) {
     res <- resolveMPI(x)
   } else {
@@ -314,6 +333,7 @@ result.ClusterFuture <- function(future, ...) {
 }
 
 
+#' @importFrom parallelly isConnectionValid
 receiveMessageFromWorker <- function(future, ...) {
   debug <- getOption("future.debug", FALSE)
   if (debug) {
@@ -339,7 +359,7 @@ receiveMessageFromWorker <- function(future, ...) {
 
   if (!is.null(con <- node$con)) {
     if (debug) mdebugf("- Validating connection of %s", class(future)[1])
-    isValid <- isValidConnection(con)
+    isValid <- isConnectionValid(con)
     if (!isValid) {
       label <- future$label
       if (is.null(label)) label <- "<none>"
@@ -415,9 +435,24 @@ receiveMessageFromWorker <- function(future, ...) {
 
   ## Non-expected message from worker?
   if (!inherits(msg, "FutureResult") && !inherits(msg, "condition")) {
-    str(list(node_idx = node_idx, node = node))
-    hint <- sprintf("This suggests that the communication with %s worker (%s #%d) is out of sync.",
-                    class(future)[1], sQuote(class(node)[1]), node_idx)
+    ## If parallel:::slaveLoop() ends up capturing the error, which should
+    ## not happen unless there is a critical error, then it'll be of captured
+    ## by try().
+    if (inherits(msg, "try-error")) {
+      ex <- FutureError(msg, future = future)
+      future$result <- ex
+      stop(ex)
+    }
+    
+    node_info <- sprintf("%s #%d", sQuote(class(node)[1]), node_idx)
+    if (inherits(node, "RichSOCKnode")) {
+      specs <- summary(node)
+      node_info <- sprintf("%s on host %s (%s, platform %s)",
+                           node_info, sQuote(specs[["host"]]),
+                           specs[["r_version"]], specs[["platform"]])
+    }
+    
+    hint <- sprintf("This suggests that the communication with %s is out of sync.", node_info)
     ex <- UnexpectedFutureResultError(future, hint = hint)
     future$result <- ex
     stop(ex)
@@ -456,10 +491,12 @@ receiveMessageFromWorker <- function(future, ...) {
       ## Cleanup global environment while at it
       if (!future$persistent) clusterCall(cl[1], fun = grmall)
       
-      ## WORKAROUND: Need to clear cluster worker before garbage collection,
-      ## cf. https://github.com/HenrikBengtsson/Wishlist-for-R/issues/27
-      ## UPDATE: This has been fixed in R (>= 3.3.2) /HB 2016-10-13
-      ## Return a value identifiable for troubleshooting purposes
+      ## WORKAROUND: Need to clear cluster worker before garbage collection.
+      ## This is needed for workers running R (<= 3.3.1). It will create
+      ## another teeny, dummy object on the worker allowing any previous
+      ## objects to be garbage collected.  For more details, see
+      ## https://github.com/HenrikBengtsson/Wishlist-for-R/issues/27.
+      ## (We return a value identifiable for troubleshooting purposes)
       clusterCall(cl[1], function() "future-clearing-cluster-worker")
       
       clusterCall(cl[1], gc, verbose = FALSE, reset = FALSE)
@@ -573,7 +610,7 @@ requestNode <- function(await, workers, timeout = getOption("future.wait.timeout
 
 
 #' @export
-getExpression.ClusterFuture <- function(future, expr = future$expr, immediateConditions = TRUE, conditionClasses = future$conditions, resignalImmediateConditions = immediateConditions, ...) {
+getExpression.ClusterFuture <- function(future, expr = future$expr, immediateConditions = TRUE, conditionClasses = future$conditions, resignalImmediateConditions = getOption("future.psock.relay.immediate", immediateConditions), ...) {
   ## Assert that no arguments but the first is passed by position
   assert_no_positional_args_but_first()
 
@@ -603,13 +640,13 @@ getExpression.ClusterFuture <- function(future, expr = future$expr, immediateCon
               if (exists("sendData", mode = "function", envir = ns)) {
                 parallel_sendData <- get("sendData", mode = "function", envir = ns)
   
-                ## Find the 'master' argument of the worker's slaveLoop()
+                ## Find the 'master' argument of the worker's {slave,work}Loop()
                 envir <- sys.frame(frame)
                 master <- NULL
                 while (!identical(envir, .GlobalEnv) && !identical(envir, emptyenv())) {
                   if (exists("master", mode = "list", envir = envir, inherits=FALSE)) {
                     master <- get("master", mode = "list", envir = envir, inherits = FALSE)
-                    if (inherits(master, "SOCKnode")) {
+                    if (inherits(master, c("SOCKnode", "SOCK0node"))) {
                       sendCondition <<- function(cond) {
                         data <- list(type = "VALUE", value = cond, success = TRUE)
                         parallel_sendData(master, data)
