@@ -23,7 +23,7 @@
 #' 
 #' @param conditions A character string of conditions classes to be captured
 #' and relayed.  The default is to relay messages and warnings.
-#' To not intercept conditions, use `conditions = character(0L)`.
+#' To not intercept any types of conditions, use `conditions = NULL`.
 #' Errors are always relayed.
 #' 
 #' @param globals (optional) a logical, a character vector, or a named list
@@ -37,15 +37,16 @@
 #' @param seed (optional) If TRUE, the random seed, that is, the state of the
 #' random number generator (RNG) will be set such that statistically sound
 #' random numbers are produced (also during parallelization).
-#' If FALSE, it is assumed that the future expression does neither need nor
-#' use random numbers generation.
+#' If FALSE (default), it is assumed that the future expression does neither
+#' need nor use random numbers generation.
 #' To use a fixed random seed, specify a L'Ecuyer-CMRG seed (seven integer)
-#' or a regular RNG seed (a single integer).
+#' or a regular RNG seed (a single integer).  If the latter, then a
+#' L'Ecuyer-CMRG seed will be automatically created based on the given seed.
 #' Furthermore, if FALSE, then the future will be monitored to make sure it
 #' does not use random numbers.  If it does and depending on the value of
-#' option \code{\link[=future.options]{future.rng.misUse}}, the check is
+#' option \code{\link[=future.options]{future.rng.onMisuse}}, the check is
 #' ignored, an informative warning, or error will be produced.
-#' If `seed` is NULL (default), then the effect is as with `seed = FALSE`
+#' If `seed` is NULL, then the effect is as with `seed = FALSE`
 #' but without the RNG check being performed.
 #'
 #' @param lazy If FALSE (default), the future is resolved
@@ -81,13 +82,13 @@
 #' An alternative approach is to use the \code{\link{\%<-\%}} infix
 #' assignment operator, which creates a future from the
 #' right-hand-side (RHS) \R expression and assigns its future value
-#' to a variable as a \emph{\link[base]{promise}}.
+#' to a variable as a \emph{\link[base:delayedAssign]{promise}}.
 #'
 #' @importFrom parallel nextRNGStream
 #' @export
 #' @keywords internal
 #' @name Future-class
-Future <- function(expr = NULL, envir = parent.frame(), substitute = FALSE, stdout = TRUE, conditions = "condition", globals = NULL, packages = NULL, seed = FALSE, lazy = FALSE, local = TRUE, gc = FALSE, earlySignal = FALSE, label = NULL, ...) {
+Future <- function(expr = NULL, envir = parent.frame(), substitute = TRUE, stdout = TRUE, conditions = "condition", globals = NULL, packages = NULL, seed = FALSE, lazy = FALSE, local = TRUE, gc = FALSE, earlySignal = FALSE, label = NULL, ...) {
   if (substitute) expr <- substitute(expr)
 
   if (is.null(seed)) {
@@ -114,8 +115,15 @@ Future <- function(expr = NULL, envir = parent.frame(), substitute = FALSE, stdo
     stop_if_not(!anyNA(packages), all(nzchar(packages)))
   }
 
+  if (!is.null(label)) {
+    stop_if_not(is.character(label))
+  }
+  
   args <- list(...)
 
+  if (!local && !isTRUE(args[["persistent"]])) {
+    .Deprecated(msg = "Using 'local = FALSE' for a future is deprecated and will soon be defunct and produce an error.", package = .packageName)
+  }
 
   core <- new.env(parent = emptyenv())
 
@@ -138,11 +146,6 @@ Future <- function(expr = NULL, envir = parent.frame(), substitute = FALSE, stdo
 
   ## Result
   core$result <- NULL
-  ## IMPORTANT: Do *not* set 'value', because there are, checks for a future
-  ## being resolved or not that checks for its existance, e.g.
-  ## exists("value", envir = future).
-  ## UPDATE: 'value' is being replaced by 'result$value' /HB 2018-02-25
-  ## core$value <- NULL
 
   ## Future miscellaneous
   core$label <- label
@@ -156,8 +159,17 @@ Future <- function(expr = NULL, envir = parent.frame(), substitute = FALSE, stdo
   core$state <- "created"
 
   ## Additional named arguments
-  for (key in names(args)) core[[key]] <- args[[key]]
-  
+  names <- names(args)
+  for (key in names) core[[key]] <- args[[key]]
+
+  ## IMPORTANT: Do *not* set 'value' because that field is defunct but
+  ## there might still be legacy code out there that rely on it.  By
+  ## assert it is not set here, it is more likely to be caught.  This
+  ## check will eventually be removed
+  if ("value" %in% names) {
+    .Defunct(msg = "Future field 'value' is defunct and must not be set", package = .packageName)
+  }
+
   structure(core, class = c("Future", class(core)))
 }
 
@@ -175,7 +187,7 @@ print.Future <- function(x, ...) {
   cat(sprintf("Lazy evaluation: %s\n", x$lazy))
   cat(sprintf("Asynchronous evaluation: %s\n", x$asynchronous))
   cat(sprintf("Local evaluation: %s\n", x$local))
-  cat(sprintf("Environment: %s\n", capture.output(x$envir)))
+  cat(sprintf("Environment: %s\n", envname(x$envir)))
   cat(sprintf("Capture standard output: %s\n", x$stdout))
   if (length(x$conditions) > 0) {
     cat(sprintf("Capture condition classes: %s\n",
@@ -239,8 +251,10 @@ print.Future <- function(x, ...) {
   if (hasResult) {
     if (inherits(result, "FutureResult")) {
       value <- result$value
+    } else if ("value" %in% names(x)) {
+      .Defunct(msg = sprintf("Detected a %s object that rely on the defunct 'value' field of format version 1.7 or before.", class(x)[1]), package = .packageName)
     } else {
-      value <- x$value
+      stop(FutureError(sprintf("The %s object does not have a 'results' field", class(x)[1]), future = future))
     }
     cat(sprintf("Value: %s of class %s\n", asIEC(objectSize(value)), sQuote(class(value)[1])))
     if (inherits(result, "FutureResult")) {
@@ -378,116 +392,10 @@ result.Future <- function(future, ...) {
     }
   }
 
-  .Defunct(msg = "Future objects with an internal version of 1.7 or earlier are defunct. This error is likely coming from a third-party package or other R code. Please report this to the maintainer of the 'future' package so this can be resolved.")
+  .Defunct(msg = "Future objects with an internal version of 1.7 or earlier are defunct. This error is likely coming from a third-party package or other R code. Please report this to the maintainer of the 'future' package so this can be resolved.", package = .packageName)
 }
 
 
-#' The value of a future
-#'
-#' Gets the value of a future.  If the future is unresolved, then
-#' the evaluation blocks until the future is resolved.
-#'
-#' @param future A \link{Future}.
-#' 
-#' @param stdout If TRUE, any captured standard output is outputted,
-#' otherwise not.
-#' 
-#' @param signal A logical specifying whether (\link[base]{conditions})
-#' should signaled or be returned as values.
-#' 
-#' @param \dots Not used.
-#'
-#' @return An \R object of any data type.
-#'
-#' @details
-#' This method needs to be implemented by the class that implement
-#' the Future API.
-#'
-#' @aliases value
-#' @rdname value
-#' @export
-#' @export value
-value.Future <- function(future, stdout = TRUE, signal = TRUE, ...) {
-  if (future$state == "created") {
-    future <- run(future)
-  }
-
-  result <- result(future)
-  stop_if_not(inherits(result, "FutureResult"))
-
-  value <- result$value
-  visible <- result$visible
-  if (is.null(visible)) visible <- TRUE
-
-  ## Always signal immediateCondition:s and as soon as possible.
-  ## They will always be signaled if they exist.
-  signalImmediateConditions(future)
-
-  ## Output captured standard output?
-  if (stdout && length(result$stdout) > 0 &&
-      inherits(result$stdout, "character")) {
-    cat(paste(result$stdout, collapse = "\n"))
-  }
-
-
-
-  ## Was RNG used without requesting RNG seeds?
-  if (!isTRUE(future$.rng_checked) && isFALSE(future$seed) && isTRUE(result$rng)) {
-    ## BACKWARD COMPATIBILITY: Until higher-level APIs set future()
-    ## argument 'seed' to indicate that RNGs are used. /HB 2019-12-24
-    ## future.apply (<= 1.3.0) and furrr
-    rng_ok <-           is_lecyer_cmrg_seed(future$globals$...future.seeds_ii[[1]])
-    rng_ok <- rng_ok || is_lecyer_cmrg_seed(future$envir$...future.seeds_ii[[1]])
-    ## doFuture w/ doRNG, e.g. %dorng%
-    rng_ok <- rng_ok || any(grepl(".doRNG.stream", deparse(future$expr), fixed = TRUE))
-    if (!rng_ok) {
-      onMisuse <- Sys.getenv("R_FUTURE_RNG_ONMISUSE", "ignore")
-      onMisuse <- getOption("future.rng.onMisuse", onMisuse)
-      if (onMisuse != "ignore") {
-        label <- future$label
-        if (is.null(label)) label <- "<none>"
-        msg <- sprintf("UNRELIABLE VALUE: Future (%s) unexpectedly generated random numbers without specifying argument '[future.]seed'. There is a risk that those random numbers are not statistically sound and the overall results might be invalid. To fix this, specify argument argument '[future.]seed', e.g. 'seed=TRUE'. This ensures that proper, parallel-safe random numbers are produced via the L'Ecuyer-CMRG method. To disable this check, set option 'future.rng.onMisuse' to \"ignore\".", sQuote(label))
-        if (onMisuse == "error") {
-          cond <- simpleError(msg)
-        } else if (onMisuse == "warning") {
-          cond <- simpleWarning(msg)
-	} else {
-	  cond <- NULL
-	  warning("Unknown value on option 'future.rng.onMisuse': ",
-                  sQuote(onMisuse))
-	}
-	conditions <- result$conditions
-	conditions[[length(conditions) + 1L]] <- list(condition = cond, signaled = FALSE)
-        result$conditions <- conditions
-	future$result <- result
-      }
-    }
-  }
-  future$.rng_checked <- TRUE
-
-  ## Signal captured conditions?
-  conditions <- result$conditions
-  if (length(conditions) > 0) {
-    if (signal) {
-      mdebugf("Future state: %s", sQuote(future$state))
-      ## Will signal an (eval) error, iff exists
-      signalConditions(future, exclude = getOption("future.relay.immediate", "immediateCondition"), resignal = TRUE)
-    } else {
-      ## Return 'error' object, iff exists, otherwise NULL
-      error <- conditions[[length(conditions)]]$condition
-      if (inherits(error, "error")) {
-        value <- error
-        visible <- TRUE
-      }
-    }
-  }
-  
-  if (visible) value else invisible(value)
-}
-
-value <- function(...) UseMethod("value")
-
-
 #' @export
 resolved.Future <- function(x, run = TRUE, ...) {
   ## A lazy future not even launched?
@@ -521,7 +429,7 @@ resolved.Future <- function(x, run = TRUE, ...) {
 #' [multicore] and [multisession] ones.
 #' The default will also set `options(mc.cores = 1L)` (*) so that
 #' no parallel \R processes are spawned off by functions such as
-#' \code{\link[parallel:mclapply]{mclapply}()} and friends.
+#' \code{parallel::mclapply()} and friends.
 #'
 #' Currently it is not possible to specify what type of nested
 #' futures to be used, meaning the above default will always be
@@ -544,10 +452,13 @@ resolved.Future <- function(x, run = TRUE, ...) {
 getExpression <- function(future, ...) UseMethod("getExpression")
 
 #' @export
-getExpression.Future <- function(future, expr = future$expr, local = future$local, stdout = future$stdout, conditionClasses = future$conditions, mc.cores = NULL, ...) {
+getExpression.Future <- function(future, expr = future$expr, local = future$local, stdout = future$stdout, conditionClasses = future$conditions, split = future$split, mc.cores = NULL, exit = NULL, ...) {
   debug <- getOption("future.debug", FALSE)
   ##  mdebug("getExpression() ...")
 
+  if (is.null(split)) split <- FALSE
+  stop_if_not(is.logical(split), length(split) == 1L, !is.na(split))
+ 
   version <- future$version
   if (is.null(version)) {
     warning(FutureWarning("Future version was not set. Using default %s",
@@ -556,30 +467,39 @@ getExpression.Future <- function(future, expr = future$expr, local = future$loca
 
 
   enter <- bquote({
-    ## covr: skip=4
-    ## If 'future' is not installed on the worker, or a too old version
-    ## of 'future' is used, then give an early error
-    ## If future::FutureResult does not exist, give an error
-    has_future <- base::requireNamespace("future", quietly = TRUE)
-    version <- if (has_future) utils::packageVersion("future") else NULL
-    if (!has_future || version < "1.8.0") {
-      info <- base::c(
-        r_version = base::gsub("R version ", "", base::R.version$version.string),
-        platform = base::sprintf("%s (%s-bit)", base::R.version$platform, 8 * base::.Machine$sizeof.pointer),
-        os = base::paste(base::Sys.info()[base::c("sysname", "release", "version")], collapse = " "),
-        hostname = base::Sys.info()[["nodename"]]
-      )
-      info <- base::sprintf("%s: %s", base::names(info), info)
-      info <- base::paste(info, collapse = "; ")
-      if (!has_future) {
-        msg <- base::sprintf("Package 'future' is not installed on worker (%s)", info)
+    base::local({
+      ## covr: skip=4
+      ## If 'future' is not installed on the worker, or a too old version
+      ## of 'future' is used, then give an early error
+      ## If future::FutureResult does not exist, give an error
+      has_future <- base::requireNamespace("future", quietly = TRUE)
+      if (has_future) {
+        ## future (>= 1.20.0)
+        ns <- base::getNamespace("future")
+        version <- ns[[".package"]][["version"]]
+        ## future (< 1.20.0)
+        if (is.null(version)) version <- utils::packageVersion("future")
       } else {
-        msg <- base::sprintf("Package 'future' on worker (%s) must be of version >= 1.8.0: %s", info, version)
+        version <- NULL
       }
-      base::stop(msg)
-    }
+      if (!has_future || version < "1.8.0") {
+        info <- base::c(
+          r_version = base::gsub("R version ", "", base::R.version$version.string),
+          platform = base::sprintf("%s (%s-bit)", base::R.version$platform, 8 * base::.Machine$sizeof.pointer),
+          os = base::paste(base::Sys.info()[base::c("sysname", "release", "version")], collapse = " "),
+          hostname = base::Sys.info()[["nodename"]]
+        )
+        info <- base::sprintf("%s: %s", base::names(info), info)
+        info <- base::paste(info, collapse = "; ")
+        if (!has_future) {
+          msg <- base::sprintf("Package 'future' is not installed on worker (%s)", info)
+        } else {
+          msg <- base::sprintf("Package 'future' on worker (%s) must be of version >= 1.8.0: %s", info, version)
+        }
+        base::stop(msg)
+      }
+    })
   })
-  exit <- NULL
   
   ## Should 'mc.cores' be set?
   if (!is.null(mc.cores)) {
@@ -594,7 +514,8 @@ getExpression.Future <- function(future, expr = future$expr, local = future$loca
     })
 
     exit <- bquote({
-      ## covr: skip=1
+      ## covr: skip=2
+      .(exit)
       base::options(mc.cores = ...future.mc.cores.old)
     })
   }
@@ -685,7 +606,7 @@ getExpression.Future <- function(future, expr = future$expr, local = future$loca
     future::plan(.(strategies), .cleanup = FALSE, .init = FALSE)
   })
 
-  expr <- makeExpression(expr = expr, local = local, stdout = stdout, conditionClasses = conditionClasses, enter = enter, exit = exit, ..., version = version)
+  expr <- makeExpression(expr = expr, local = local, stdout = stdout, conditionClasses = conditionClasses, split = split, enter = enter, exit = exit, ..., version = version)
   if (getOption("future.debug", FALSE)) mprint(expr)
 
 ##  mdebug("getExpression() ... DONE")
@@ -697,9 +618,8 @@ getExpression.Future <- function(future, expr = future$expr, local = future$loca
 makeExpression <- local({
   skip <- skip.local <- NULL
   
-  function(expr, local = TRUE, immediateConditions = FALSE, stdout = TRUE, conditionClasses = NULL, globals.onMissing = getOption("future.globals.onMissing", "ignore"), enter = NULL, exit = NULL, version = "1.8") {
-    if (is.null(conditionClasses)) conditionClasses <- character(0L)
-    if (immediateConditions) {
+  function(expr, local = TRUE, immediateConditions = FALSE, stdout = TRUE, conditionClasses = NULL, split = FALSE, globals.onMissing = getOption("future.globals.onMissing", NULL), enter = NULL, exit = NULL, version = "1.8") {
+    if (immediateConditions && !is.null(conditionClasses)) {
       immediateConditionClasses <- getOption("future.relay.immediate", "immediateCondition")
       conditionClasses <- unique(c(conditionClasses, immediateConditionClasses))
     } else {
@@ -740,6 +660,7 @@ makeExpression <- local({
         future.globals.onReference = .(getOption("future.globals.onReference")),
         future.globals.resolve     = .(getOption("future.globals.resolve")),
         future.resolve.recursive   = .(getOption("future.resolve.recursive")),
+        future.rng.onMisuse        = .(getOption("future.rng.onMisuse")),
         
         ## Other options relevant to making futures behave consistently
         ## across backends
@@ -765,7 +686,7 @@ makeExpression <- local({
         .(enter)
   
         ## Capture standard output?
-        if (base::is.na(.(stdout))) {  ## stdout = NA
+        if (.(base::is.na(stdout))) {  ## stdout = NA
           ## Don't capture, but also don't block any output
         } else {
           if (.(stdout)) {  ## stdout = TRUE
@@ -781,9 +702,9 @@ makeExpression <- local({
               open = "w"
             )
           }
-          base::sink(...future.stdout, type = "output", split = FALSE)
+          base::sink(...future.stdout, type = "output", split = .(split))
           base::on.exit(if (!base::is.null(...future.stdout)) {
-            base::sink(type = "output", split = FALSE)
+            base::sink(type = "output", split = .(split))
             base::close(...future.stdout)
           }, add = TRUE)
         }
@@ -794,7 +715,6 @@ makeExpression <- local({
         ...future.result <- base::tryCatch({
           base::withCallingHandlers({
             ...future.value <- base::withVisible(.(expr))
-            ## A FutureResult object (without requiring the future package)
             future::FutureResult(value = ...future.value$value, visible = ...future.value$visible, rng = !identical(base::globalenv()$.Random.seed, ...future.rng), started = ...future.startTime, version = "1.8")
           }, condition = base::local({
               ## WORKAROUND: If the name of any of the below objects/functions
@@ -812,7 +732,6 @@ makeExpression <- local({
               seq.int <- base::seq.int
               signalCondition <- base::signalCondition
               sys.calls <- base::sys.calls
-              Sys.time <- base::Sys.time
               `[[` <- base::`[[`
               `+` <- base::`+`
               `<<-` <- base::`<<-`
@@ -820,19 +739,49 @@ makeExpression <- local({
               sysCalls <- function(calls = sys.calls(), from = 1L) {
                 calls[seq.int(from = from + .(skip[1L]), to = length(calls) - .(skip[2L]))]
               }
-  
               function(cond) {
                 ## Handle error:s specially
                 if (inherits(cond, "error")) {
-                  ...future.conditions[[length(...future.conditions) + 1L]] <<- list(condition = cond, calls = c(sysCalls(from = ...future.frame), cond$call), timestamp = Sys.time(), signaled = 0L)
+                  sessionInformation <- function() {
+                    list(
+                      r          = base::R.Version(),
+                      locale     = base::Sys.getlocale(),
+		      rngkind    = base::RNGkind(),
+		      namespaces = base::loadedNamespaces(),
+		      search     = base::search(),
+		      system     = base::Sys.info()
+		    )
+                  }
+
+                  ## Record condition
+                  ...future.conditions[[length(...future.conditions) + 1L]] <<- list(
+                    condition = cond,
+                    calls     = c(sysCalls(from = ...future.frame), cond$call),
+                    session   = sessionInformation(),
+                    timestamp = base::Sys.time(),
+                    signaled  = 0L
+                  )
+		  
                   signalCondition(cond)
-                } else if (inherits(cond, .(conditionClasses))) {
+                } else if (.(!is.null(conditionClasses)) &&
+                           inherits(cond, .(conditionClasses))) {
                   ## Relay 'immediateCondition' conditions immediately?
                   ## If so, then do not muffle it and flag it as signalled
                   ## already here.
                   signal <- .(immediateConditions) && inherits(cond, .(immediateConditionClasses))
-                  ...future.conditions[[length(...future.conditions) + 1L]] <<- list(condition = cond, signaled = base::as.integer(signal))
-                  if (!signal) {
+                  ## Record condition
+                  ...future.conditions[[length(...future.conditions) + 1L]] <<- list(
+		    condition = cond,
+		    signaled = base::as.integer(signal)
+		  )
+                  if (.(immediateConditions && !split) && !signal) {
+                    ## muffleCondition <- future:::muffleCondition()
+                    muffleCondition <- .(muffleCondition)
+                    muffleCondition(cond)
+                  }
+                } else {
+                  if (.(!split && !is.null(conditionClasses))) {
+                    ## Muffle all non-captured conditions
                     ## muffleCondition <- future:::muffleCondition()
                     muffleCondition <- .(muffleCondition)
                     muffleCondition(cond)
@@ -850,10 +799,9 @@ makeExpression <- local({
             version = "1.8"
           ), class = "FutureResult")
         }, finally = .(exit))
-        Sys.time
-        if (base::is.na(.(stdout))) {
-        } else {
-          base::sink(type = "output", split = FALSE)
+	
+        if (.(!base::is.na(stdout))) {
+          base::sink(type = "output", split = .(split))
           if (.(stdout)) {
             ...future.result$stdout <- base::rawToChar(
               base::rawConnectionValue(...future.stdout)
@@ -869,16 +817,6 @@ makeExpression <- local({
         
         ...future.result
       })
-    } else if (version == "1.7") {  ## Deprecated
-      expr <- bquote({
-        ## covr: skip=6
-        .(enter)
-        base::tryCatch({
-          .(expr)
-        }, finally = {
-          .(exit)
-        })
-      })
     } else {
       stop(FutureError("Internal error: Non-supported future expression version: ", version))
     }
