@@ -13,9 +13,10 @@
 #' @export
 #' @name MultiprocessFuture-class
 #' @keywords internal
-MultiprocessFuture <- function(expr = NULL, envir = parent.frame(), substitute = FALSE, ...) {
+MultiprocessFuture <- function(expr = NULL, substitute = TRUE, envir = parent.frame(), ...) {
   if (substitute) expr <- substitute(expr)
 
-  f <- Future(expr = expr, envir = envir, substitute = FALSE, ...)
-  structure(f, class = c("MultiprocessFuture", class(f)))
+  future <- Future(expr = expr, envir = envir, substitute = FALSE, ...)
+  future <- structure(future, class = c("MultiprocessFuture", class(future)))
+  future
 }
