@@ -1,5 +1,7 @@
 #' Create a future
 #'
+## Rescaled to 120x138: convert -geometry 28.7% future-logo.png logo.png
+#' \if{html}{\figure{logo.png}{options: align='right' alt='logo' width='120'}}
 #' Creates a future that evaluates an \R expression or
 #' a future that calls an \R function with a set of arguments.
 #' How, when, and where these futures are evaluated can be configured
@@ -14,7 +16,7 @@
 #' 
 #' @param expr,value An \R \link[base]{expression}.
 #'
-#' @param \dots Reserved for internal use only.
+#' @param \dots Additional arguments passed to [Future()].
 #'
 #' @return
 #' `f <- future(expr)` creates a [Future] `f` that evaluates expression `expr`, the value of the future is retrieved using `v <- value(f)`.
@@ -172,6 +174,8 @@
 #'
 #' @example incl/future.R
 #'
+#' @author
+#' The future logo was designed by Dan LaBar and tweaked by Henrik Bengtsson.
 #'
 #' @seealso
 #' How, when and where futures are resolved is given by the
@@ -182,35 +186,43 @@
 #' @aliases futureCall
 #' @export
 #' @name future
-future <- function(expr, envir = parent.frame(), substitute = TRUE, globals = TRUE, packages = NULL, seed = FALSE, lazy = FALSE, ...) {
+future <- function(expr, envir = parent.frame(), substitute = TRUE, lazy = FALSE, seed = FALSE, globals = TRUE, packages = NULL, label = NULL, gc = FALSE, ...) {
   if (substitute) expr <- substitute(expr)
 
-  ## Hidden argument 'evaluator':
-  ## The 'evaluator' is the function that creates a Future object.
-  ## The default evaluator function is given by plan().
-  makeFuture <- list(...)$evaluator
-  if (!is.null(makeFuture)) {
-    action <- get(Sys.getenv("R_CHECK_FUTURE_EVALUATOR", ".Defunct"), mode="function")
-    action(msg = "Argument 'evaluator' of future() was an internal argument and is now deprecated. Use plan() to set the \"evaluator\".")
-  }
-  if (is.null(makeFuture)) makeFuture <- plan("next")
-  if (!is.function(makeFuture)) {
-    stop("Argument 'evaluator' must be a function: ", typeof(makeFuture))
-  }
-
-  ## Call 'makeFuture' without passing 'evaluator' argument part of '...'
-  .makeFuture <- function(..., evaluator = NULL) makeFuture(...)
-  future <- .makeFuture(expr, substitute = FALSE,
-                        envir = envir,
-                        globals = globals, packages = packages,
-                        seed = seed,
-                        lazy = lazy,
-                        ...)
+  makeFuture <- plan("next")
+  future <- makeFuture(expr, substitute = FALSE,
+                       envir = envir,
+                       lazy = lazy,
+                       seed = seed,
+                       globals = globals,
+                       packages = packages,
+                       label = label,
+                       gc = gc,
+                       ...)
 
   ## Assert that a future was returned
   if (!inherits(future, "Future")) {
-    stop(FutureError("Argument 'evaluator' specifies a function that does not return a Future object: ", paste(sQuote(class(future)), collapse = ", ")))
+    stop(FutureError("plan(\"next\") returned a function that does not return a Future object: ", paste(sQuote(class(future)), collapse = ", ")))
   }
 
   future
 }
+
+## Arguments to 'future' strategies that must not be tweaked
+attr(future, "untweakable") <- c(
+  "asynchronous",  ## reserved
+  "conditions",
+  "envir",
+  "expr",
+  "globals",
+  "lazy",
+  "local",
+  "packages",
+  "seed",
+  "stdout",
+  "substitute",
+  "version"        ## for internal backend use
+)
+
+## Hidden arguments to 'future' strategy that my also be tweaked
+attr(future, "tweakable") <- c("earlySignal", "split")
