@@ -9,8 +9,12 @@ isNA <- function(x) {
 assert_no_positional_args_but_first <- function(call = sys.call(sys.parent())) {
   ast <- as.list(call)
   if (length(ast) <= 2L) return()
-  names <- names(ast[-(1:2)])
-  if (is.null(names) || any(names == "")) {
+  ast <- ast[-(1:2)]
+  dots <- vapply(ast, FUN = identical, as.symbol("..."), FUN.VALUE = FALSE)
+  ast <- ast[!dots]
+  if (length(ast) == 0L) return()
+  names <- names(ast)
+  if (is.null(names) || any(names == "")) {    
     stop(sprintf("Function %s() requires that all arguments beyond the first one are passed by name and not by position: %s", as.character(call[[1L]]), deparse(call, width.cutoff = 100L)))
   }
 }
@@ -93,6 +97,34 @@ asIEC <- function(size, digits = 2L) {
   sprintf(fmt, size)
 } # asIEC()
 
+#' @importFrom utils capture.output
+envname <- function(env) {
+  if (!is.environment(env)) return(NA_character_)
+  name <- environmentName(env)
+  if (name == "") {
+    class <- class(env)
+    if (identical(class, "environment")) {
+      ## e.g. new.env()
+      name <- capture.output(print(env))
+    } else {
+      ## It might be that 'env' is on a class that extends 'environment',
+      ## e.g. R.oo::Object() or R6::R6Class().
+      ## IMPORTANT: The unset class must be temporary, because changing
+      ## the class of an environment will
+      name <- local({
+        on.exit(class(env) <- class)
+        class(env) <- NULL
+        capture.output(print(env))
+      })	
+    }
+    if (length(name) > 1L) name <- name[1]
+    name <- gsub("(.*: |>)", "", name)
+  } else {
+    ## e.g. globals:::where("plan")
+    name <- gsub("package:", "", name, fixed = TRUE)
+  }
+  name
+}
 
 now <- function(x = Sys.time(), format = "[%H:%M:%OS3] ") {
   ## format(x, format = format) ## slower
@@ -193,36 +225,6 @@ getOption <- local({
 }) ## getOption()
 
 
-detectCores <- local({
-  res <- NULL
-  function() {
-    if (is.null(res)) {
-      ## Get number of system cores from option, system environment,
-      ## and finally detectCores().  This also designed such that
-      ## it is indeed possible to return NA_integer_.
-      value <- getOption("future.availableCores.system")
-      if (!is.null(value)) {
-        value <- as.integer(value)
-        return(value)
-      }
-      
-      value <- parallel::detectCores()
-      
-      ## If unknown, set default to 1L
-      if (is.na(value)) value <- 1L
-      value <- as.integer(value)
-      
-      ## Assert positive integer
-      stop_if_not(length(value) == 1L, is.numeric(value),
-                is.finite(value), value >= 1L)
-
-      res <<- value
-    }
-    res
-  }
-})
-
-
 ## We are currently importing the following non-exported functions:
 ## * cluster futures:
 ##   - parallel:::defaultCluster()  ## non-critical / not really needed /
@@ -694,7 +696,7 @@ objectSize <- function(x, depth = 3L, enclosure = getOption("future.globals.obje
 
 #' Creates a connection to the system null device
 #'
-#' @return Returns a open, binary \code{\link[base:connection]{base::connection()}}.
+#' @return Returns a open, binary \code{\link[base:connections]{base::connection()}}.
 #' 
 #' @keywords internal
 nullcon <- local({
@@ -881,7 +883,7 @@ resolveMPI <- local({
           }
         }
       }
-      stopifnot(is.function(resolveMPI))
+      stop_if_not(is.function(resolveMPI))
       cache$resolveMPI <<- resolveMPI
     }
 
@@ -912,7 +914,7 @@ resolveMPI <- local({
 #' @references
 #' 1. The Open Group Base Specifications Issue 7, 2018 edition,
 #'    IEEE Std 1003.1-2017 (Revision of IEEE Std 1003.1-2008)
-#'    \url{http://pubs.opengroup.org/onlinepubs/9699919799/functions/kill.html}
+#'    \url{https://pubs.opengroup.org/onlinepubs/9699919799/functions/kill.html}
 #'
 #' 2. Microsoft, tasklist, 2018-08-30,
 #'    \url{https://docs.microsoft.com/en-us/windows-server/administration/windows-commands/tasklist}
@@ -1117,34 +1119,13 @@ pid_exists <- local({
 })
 
 
-#' @importFrom tools pskill
-pid_kill <- function(pid, wait = 0.5, timeout = 30, debug = TRUE) {
-  pid <- as.integer(pid)
-  stop_if_not(length(pid), !is.na(pid), pid >= 0L)
-
-  setTimeLimit(elapsed = timeout)
-  on.exit(setTimeLimit(elapsed = Inf))
-
-  tryCatch({
-    ## Always try to kill, because pid_exists() can be very slow on Windows
-    pskill(pid)
-  
-    ## Wait a bit before checking whether process was successfully
-    ## killed or not
-    Sys.sleep(wait)
-
-    ## WARNING: pid_exists() can be very slow on Windows
-    !isTRUE(pid_exists(pid))
-  }, error = function(ex) NA)
-}
-
-
 ## From R.utils 2.7.0 (2018-08-26)
 queryRCmdCheck <- function(...) {
   evidences <- list()
 
   # Command line arguments
   args <- commandArgs()
+
   evidences[["vanilla"]] <- is.element("--vanilla", args)
 
   # Check the working directory
@@ -1155,16 +1136,28 @@ queryRCmdCheck <- function(...) {
 
   # Is 'R CMD check' checking tests?
   evidences[["tests"]] <- (
-    (regexpr(pattern, parent) != -1) && 
-    (regexpr("^tests(|_.*)$", dirname) != -1)
+    grepl(pattern, parent) && grepl("^tests(|_.*)$", dirname)
   )
 
   # Is the current working directory as expected?
-  evidences[["pwd"]] <- (evidences[["tests"]] || (regexpr(pattern, dirname) != -1))
+  evidences[["pwd"]] <- (evidences[["tests"]] || grepl(pattern, dirname))
 
   # Is 'R CMD check' checking examples?
   evidences[["examples"]] <- is.element("CheckExEnv", search())
+  
+  # SPECIAL: win-builder?
+  evidences[["win-builder"]] <- (.Platform$OS.type == "windows" && grepl("Rterm[.]exe$", args[1]))
 
+  if (evidences[["win-builder"]]) {
+    n <- length(args)
+    if (all(c("--no-save", "--no-restore", "--no-site-file", "--no-init-file") %in% args)) {
+      evidences[["vanilla"]] <- TRUE
+    }
+
+    if (grepl(pattern, parent)) {
+      evidences[["pwd"]] <- TRUE
+    }
+  }
 
   if (!evidences$vanilla || !evidences$pwd) {
     res <- "notRunning"
@@ -1176,6 +1169,8 @@ queryRCmdCheck <- function(...) {
     res <- "notRunning"
   }
 
+  attr(res, "evidences") <- evidences
+  
   res
 }
 
@@ -1193,7 +1188,8 @@ supports_omp_threads <- function(assert = FALSE, debug = getOption("future.debug
   ## Current number of OpenMP threads
   old_omp_threads <- RhpcBLASctl::omp_get_max_threads()
 
-  ## RhpcBLASctl compiled without OpenMP support? Then it returns NULL
+  ## RhpcBLASctl compiled without OpenMP support?
+  ## Then it returns NULL in RhpcBLASctl (< 0.20-17) otherwise NA_integer_
   if (is.null(old_omp_threads)) old_omp_threads <- NA_integer_
   
   res <- !is.na(old_omp_threads)
@@ -1202,3 +1198,52 @@ supports_omp_threads <- function(assert = FALSE, debug = getOption("future.debug
 
   res
 }
+
+
+
+## base::bquote() gained argument 'splice' in R 4.0.0 (April 2020)
+## Below is a verbatim copy of bquote() in R 4.0.3
+if (getRversion() < "4.0.0") {
+  bquote <- function(expr, where = parent.frame(), splice = FALSE) {
+      if (!is.environment(where)) 
+          where <- as.environment(where)
+      unquote <- function(e) {
+          if (is.pairlist(e)) 
+              as.pairlist(lapply(e, unquote))
+          else if (is.call(e)) {
+              if (is.name(e[[1L]]) && as.character(e[[1]]) == ".") 
+                  eval(e[[2L]], where)
+              else if (splice) {
+                  if (is.name(e[[1L]]) && as.character(e[[1L]]) == 
+                    "..") 
+                    stop("can only splice inside a call", call. = FALSE)
+                  else as.call(unquote.list(e))
+              }
+              else as.call(lapply(e, unquote))
+          }
+          else e
+      }
+      is.splice.macro <- function(e) is.call(e) && is.name(e[[1L]]) && 
+          as.character(e[[1L]]) == ".."
+      unquote.list <- function(e) {
+          p <- Position(is.splice.macro, e, nomatch = NULL)
+          if (is.null(p)) 
+              lapply(e, unquote)
+          else {
+              n <- length(e)
+              head <- if (p == 1) 
+                  NULL
+              else e[1:(p - 1)]
+              tail <- if (p == n) 
+                  NULL
+              else e[(p + 1):n]
+              macro <- e[[p]]
+              mexp <- eval(macro[[2L]], where)
+              if (!is.vector(mexp)) 
+                  stop("can only splice vectors")
+              c(lapply(head, unquote), mexp, as.list(unquote.list(tail)))
+          }
+      }
+      unquote(substitute(expr))
+  }
+}
