@@ -1,7 +1,7 @@
 #' Plan how to resolve a future
 #'
-#' This function allows \emph{the user} to plan the future, more specifically,
-#' it specifies how \code{\link{future}()}:s are resolved,
+#' This function allows _the user_ to plan the future, more specifically,
+#' it specifies how [future()]:s are resolved,
 #' e.g. sequentially or in parallel.
 #'
 #' @param strategy The evaluation function (or name of it) to use
@@ -10,15 +10,15 @@
 #' @param \dots Additional arguments overriding the default arguments
 #' of the evaluation function.  Which additional arguments are supported
 #' depends on what evaluation function is used, e.g. several support
-#' argument \code{workers} but not all.  For details, see the individual
+#' argument `workers` but not all.  For details, see the individual
 #' functions of which some are linked to below.
 #"
-#' @param substitute If TRUE, the \code{strategy} expression is
-#' \code{substitute()}:d, otherwise not.
+#' @param substitute If TRUE, the `strategy` expression is
+#' `substitute()`:d, otherwise not.
 #'
 #' @param .call (internal) Used for recording the call to this function.
 #'
-#' @param .skip (internal) If \code{TRUE}, then attempts to set a strategy
+#' @param .skip (internal) If `TRUE`, then attempts to set a strategy
 #' that is the same as what is currently in use, will skipped.
 #'
 #' @param .cleanup (internal) Used to stop implicitly started clusters.
@@ -31,39 +31,39 @@
 #' @example incl/plan.R
 #'
 #' @details
-#' The default strategy is \code{\link{sequential}}, but the default can be
+#' The default strategy is [`sequential`], but the default can be
 #' configured by option \option{future.plan} and, if that is not set,
 #' system environment variable \env{R_FUTURE_PLAN}.
-#' To reset the strategy back to the default, use \code{plan("default")}.
+#' To reset the strategy back to the default, use `plan("default")`.
 #'
 #' @section Implemented evaluation strategies:
 #' \itemize{
-#'  \item{\code{\link{sequential}}:}{
+#'  \item{[`sequential`]:}{
 #'    Resolves futures sequentially in the current \R process.
 #'  }
-#'  \item{\code{\link{transparent}}:}{
+#'  \item{[`transparent`]:}{
 #'    Resolves futures sequentially in the current \R process and
 #'    assignments will be done to the calling environment.
 #'    Early stopping is enabled by default.
 #'  }
-#'  \item{\code{\link{multisession}}:}{
+#'  \item{[`multisession`]:}{
 #'    Resolves futures asynchronously (in parallel) in separate
 #'    \R sessions running in the background on the same machine.
 #'  }
-#'  \item{\code{\link{multicore}}:}{
+#'  \item{[`multicore`]:}{
 #'    Resolves futures asynchronously (in parallel) in separate
-#'    \emph{forked} \R processes running in the background on
+#'    _forked_ \R processes running in the background on
 #'    the same machine.  Not supported on Windows.
 #'  }
-#'  \item{\code{\link{multiprocess}}:}{
+#'  \item{[`multiprocess`]:}{(DEPRECATED)
 #'    If multicore evaluation is supported, that will be used,
 #'    otherwise multisession evaluation will be used.
 #'  }
-#'  \item{\code{\link{cluster}}:}{
+#'  \item{[`cluster`]:}{
 #'    Resolves futures asynchronously (in parallel) in separate
 #'    \R sessions running typically on one or more machines.
 #'  }
-#'  \item{\code{\link{remote}}:}{
+#'  \item{[`remote`]:}{
 #'    Resolves futures asynchronously in a separate \R session
 #'    running on a separate machine, typically on a different
 #'    network.
@@ -77,39 +77,50 @@
 #' (HPC) clusters, e.g. LSF, Slurm, TORQUE/PBS, Sun Grid Engine,
 #' and OpenLava.
 #'
-#' To "close" any background workers (e.g. \code{multisession}), change
-#' the plan to something different; \code{plan(sequential)} is recommended
+#' To "close" any background workers (e.g. `multisession`), change
+#' the plan to something different; `plan(sequential)` is recommended
 #' for this.
 #'
 #' @section For package developers:
 #' Please refrain from modifying the future strategy inside your packages /
-#' functions, i.e. do not call \code{plan()} in your code.  Instead, leave
+#' functions, i.e. do not call `plan()` in your code.  Instead, leave
 #' the control on what backend to use to the end user.  This idea is part of
 #' the core philosophy of the future framework - as a developer you can never
 #' know what future backends the user have access to.  Moreover, by not making
 #' any assumptions about what backends are available, your code will also work
-#' automatically will any new backends developed after you wrote your code.
+#' automatically with any new backends developed after you wrote your code.
 #'
 #' If you think it is necessary to modify the future strategy within a
 #' function, then make sure to undo the changes when exiting the function.
 #' This can be done using:
+#'
 #' \preformatted{
-#'   oplan <- plan()
+#'   oplan <- plan(new_set_of_strategies)
 #'   on.exit(plan(oplan), add = TRUE)
 #'   [...]
 #' }
 #'
+#' This is important because the end-user might have already set the future
+#' strategy elsewhere for other purposes and will most likely not known that
+#' calling your function will break their setup.
+#' _Remember, your package and its functions might be used in a greater
+#' context where multiple packages and functions are involved and those might
+#' also rely on the future framework, so it is important to avoid stepping on 
+#' others' toes._
+#'
 #' @section Using plan() in scripts and vignettes:
 #' When writing scripts or vignettes that uses futures, try to place any
-#' call to \code{plan()} as far up (as early on) in the code as possible.  
+#' call to `plan()` as far up (as early on) in the code as possible.  
 #' This will help users to quickly identify where the future plan is set up
 #' and allow them to modify it to their computational resources.
-#' Even better is to leave it to the user to set the \code{plan()} prior to
-#' \code{source()}:ing the script or running the vignette.
+#' Even better is to leave it to the user to set the `plan()` prior to
+#' `source()`:ing the script or running the vignette.
 #' If a \file{\link{.future.R}} exists in the current directory and / or in
 #' the user's home directory, it is sourced when the \pkg{future} package is
-#' \emph{loaded}.  Because of this, the \file{.future.R} file provides a
-#' convenient place for users to set the \code{plan()}.
+#' _loaded_.  Because of this, the \file{.future.R} file provides a
+#' convenient place for users to set the `plan()`.
+#' This behavior can be controlled via an \R option - see
+#' \link[future:future.options]{future options} for more details.
 #'
 #' @importFrom utils capture.output
 #' @export
@@ -122,6 +133,32 @@ plan <- local({
   ## Stack of type of futures to use
   stack <- defaultStack
 
+  warn_about_multiprocess <- local({
+    .warn <- TRUE
+
+    function(stack) {
+      if (!.warn) return()
+
+      ## Is deprecated 'multiprocess' used?    
+      for (kk in seq_along(stack)) {
+        evaluator <- stack[[kk]]
+        if (inherits(evaluator, "multiprocess") && 
+            class(evaluator)[1] == "multiprocess") {  ## <== sic!
+          ignore <- Sys.getenv("R_FUTURE_DEPRECATED_IGNORE", "")
+          ignore <- getOption("future.deprecated.ignore", ignore)
+          ignore <- unlist(strsplit(ignore, split="[, ]", fixed = FALSE))
+          if (!is.element("multiprocess", ignore)) {
+            ## Warn only once
+            .warn <<- FALSE
+            .Deprecated(msg = sprintf("Strategy 'multiprocess' is deprecated in future (>= 1.20.0). Instead, explicitly specify either 'multisession' or 'multicore'. In the current R session, 'multiprocess' equals '%s'.", if (supportsMulticore()) "multicore" else "multisession"), package = .packageName)
+          }
+
+          break
+        }
+      }
+    }
+  })
+
   plan_cleanup <- function() {
     ClusterRegistry(action = "stop")
   }
@@ -132,9 +169,9 @@ plan <- local({
     if (identical(init, TRUE)) {
       debug <- getOption("future.debug", FALSE)
       if (debug) {
-        mdebug("plan(): plan_init() of %s ...",
-               paste(sQuote(class(evaluator)), collapse = ", "))
-        mdebug(paste(capture.output(print(evaluator)), collapse = "\n"))
+        mdebugf("plan(): plan_init() of %s ...",
+                paste(sQuote(class(evaluator)), collapse = ", "))
+        mprint(evaluator)
       }
 
       ## IMPORANT: Initiate only once.  This avoids an infinite
@@ -162,8 +199,8 @@ plan <- local({
       }
       
       if (debug) {
-        mdebug("plan(): plan_init() of %s ... DONE",
-               paste(sQuote(class(evaluator)), collapse = ", "))
+        mdebugf("plan(): plan_init() of %s ... DONE",
+                paste(sQuote(class(evaluator)), collapse = ", "))
       }
     }
   }
@@ -190,17 +227,19 @@ plan <- local({
     ## Skip if already set?
     if (skip && equal_strategy_stacks(newStack, oldStack)) {
       if (getOption("future.debug", FALSE)) {
-        mdebug(paste0("plan(): Skip setting new future strategy stack because it is the same as the current one:\n", 
-               paste(capture.output(print(newStack)), collapse = "\n"), "\n"))
+        mdebug("plan(): Skip setting new future strategy stack because it is the same as the current one:")
+        mprint(newStack)
       }
       return(oldStack)
     }
 
     if (getOption("future.debug", FALSE)) {
-      mdebug(paste0("plan(): Setting new future strategy stack:\n", 
-             paste(capture.output(print(newStack)), collapse = "\n"), "\n"))
+      mdebug("plan(): Setting new future strategy stack:")
+      mprint(newStack)
     }
-    
+
+    warn_about_multiprocess(newStack)
+
     stack <<- newStack
 
     ## Stop any (implicitly started) clusters?
@@ -212,7 +251,7 @@ plan <- local({
     ## Sanity checks
     nbrOfWorkers <- nbrOfWorkers()
     if (getOption("future.debug", FALSE)) {
-      mdebug(sprintf("plan(): nbrOfWorkers() = %g", nbrOfWorkers))
+      mdebugf(sprintf("plan(): nbrOfWorkers() = %g", nbrOfWorkers))
     }
     stop_if_not(is.numeric(nbrOfWorkers), length(nbrOfWorkers) == 1L, 
                 !is.na(nbrOfWorkers), nbrOfWorkers >= 1L)
@@ -235,6 +274,7 @@ plan <- local({
       if (!inherits(strategy, "FutureStrategy")) {
         class(strategy) <- c("FutureStrategy", class(strategy))
       }
+      stop_if_not(is.function(strategy))
       return(strategy)
     } else if (identical(strategy, "default")) {
       strategy <- getOption("future.plan", sequential)
@@ -270,14 +310,24 @@ plan <- local({
 
     if (is.list(strategy)) {
       oldStack <- plan_set(strategy, skip = .skip, cleanup = .cleanup, init = .init)
-      return(invisible(oldStack[[1L]]))
+      return(invisible(oldStack))
     }
 
     ## (a) Is a (plain) list of future strategies specified?
     if (is.language(strategy)) {
       first <- as.list(strategy)[[1]]
       if (is.symbol(first)) {
-        first <- eval(first, envir = parent.frame(), enclos = baseenv())
+        ## If a function call, e.g. list(...), then make sure to look up
+        ## a function to be used as 'first'.  This makes sure that base::list()
+        ## is used with plan(list(...)) even when there is a non-function 
+        ## 'list' on the search() path, e.g. gsubfn::list.
+        if (is.call(strategy)) {
+          first <- get(as.character(first), mode="function", 
+                       envir = parent.frame(), inherits = TRUE)
+        } else {
+          first <- eval(first, envir = parent.frame(), enclos = baseenv())
+        }
+
         ## A list object, e.g. plan(oplan)?
         if (is.list(first)) {
           strategies <- first
@@ -286,22 +336,29 @@ plan <- local({
           return(invisible(res))
         }
 
-        ## Example: plan(list(sequential, multicore))
-        if (is.function(first) && identical(first, list)) {
-          ## Specified explicitly using plan(list(...))?
+        if (is.function(first) && !inherits(first, "future")) {
           strategies <- eval(strategy, envir = parent.frame(), enclos = baseenv())
-          stop_if_not(is.list(strategies), length(strategies) >= 1L)
-          ## Coerce strings to functions, e.g.
-          ## plan(list("sequential", multicore))
-          for (kk in seq_along(strategies)) {
-            strategy_kk <- strategies[[kk]]
-            if (is.character(strategy_kk)) {
-              strategy_kk <- tweak(strategy_kk, penvir = parent.frame())
-              strategies[[kk]] <- strategy_kk
+
+          ## Specified explicitly using plan(list(...))?
+          ## Example: plan(list(sequential, multicore))
+          if (is.list(strategies)) {
+            ## Coerce strings to functions, e.g.
+            ## plan(list("sequential", multicore))
+            for (kk in seq_along(strategies)) {
+              strategy_kk <- strategies[[kk]]
+              if (is.character(strategy_kk)) {
+                strategy_kk <- tweak(strategy_kk, penvir = parent.frame())
+                strategies[[kk]] <- strategy_kk
+              }
             }
+            newStack <- strategies
+            stop_if_not(!is.null(newStack), is.list(newStack), length(newStack) >= 1L)
+          } else if (is.function(strategies) && !inherits(strategies, "future")) {
+            ## Example: plan(x$abc)
+            strategies <- list(strategies)
+            newStack <- strategies
+            stop_if_not(!is.null(newStack), is.list(newStack), length(newStack) >= 1L)
           }
-          newStack <- strategies
-          stop_if_not(!is.null(newStack), is.list(newStack), length(newStack) >= 1L)
         }
       }
     }
@@ -354,7 +411,7 @@ plan <- local({
 
     ## Set new strategy for futures
     oldStack <- plan_set(newStack, skip = .skip, cleanup = .cleanup, init = .init)
-    invisible(oldStack[[1L]])
+    invisible(oldStack)
   } # function()
 }) # plan()
 
@@ -367,7 +424,7 @@ supportedStrategies <- function(strategies = c("sequential", "multicore",
 }
 
 
-#' @importFrom utils capture.output
+#' @importFrom utils capture.output str
 #' @export
 print.future <- function(x, ...) {
   class <- setdiff(class(x), c("FutureStrategy", "tweaked", "function"))
@@ -451,3 +508,36 @@ print.FutureStrategyList <- function(x, ...) {
   cat(s, "\n", sep = "")
   invisible(x)
 }
+
+
+#' Free up active background workers
+#'
+#' @param x A FutureStrategy.
+#'
+#' @param \ldots Not used.
+#'
+#' @export
+#'
+#' @details
+#' This function will resolve any active futures that is currently
+#' being evaluated on background workers.
+#'
+#' @examples
+#' resetWorkers(plan())
+#'
+#' @keywords internal
+#' @export
+resetWorkers <- function(x, ...) UseMethod("resetWorkers")
+
+
+#' @export
+resetWorkers.default <- function(x, ...) invisible(x)
+
+#' @export
+resetWorkers.multicore <- function(x, ...) {
+  if (usedCores() == 0L) return(invisible(x))
+  reg <- sprintf("multicore-%s", session_uuid())
+  FutureRegistry(reg, action = "collect-all", earlySignal = FALSE)
+  stop_if_not(usedCores() == 0L)
+}
+
